@@ -53,9 +53,9 @@ class Pipeline:
         new_commands = self.commands[0:-1] + (new_last,)
         return dataclasses.replace(self, commands=new_commands)
 
-    def _set_write_mode(self):
-        "Set write_mode=True on last command of the pipeline."
-        new_last = self.commands[-1].set(write_mode=True)
+    def _set(self, **kwds):
+        "Set options on last command of the pipeline."
+        new_last = self.commands[-1].set(**kwds)
         new_commands = self.commands[0:-1] + (new_last,)
         return dataclasses.replace(self, commands=new_commands)
 
@@ -94,11 +94,9 @@ class Pipeline:
         return self.commands[key]
 
     def __call__(self, *args):
-        if len(args) == 0:
-            return self
-        # Use context from first command.
-        context = self.commands[0].options.context
-        return context._pipe_apply(self, args)
+        if args:
+            raise TypeError("Calling pipeline with 1 or more arguments.")
+        return self
 
     def __or__(self, rhs):
         if isinstance(rhs, (shellous.Command, Pipeline)):
@@ -117,9 +115,15 @@ class Pipeline:
             return self.stdout(rhs, append=True)
         return NotImplemented
 
-    def __invert__(self):
-        "Set write_mode=True option on last command of pipeline."
-        return self._set_write_mode()
+    @property
+    def writable(self):
+        "Set writable=True option on last command of pipeline."
+        return self._set(writable=True)
+
+    @property
+    def result(self):
+        "Set `return_result` and `exit_codes`."
+        return self._set(return_result=True, exit_codes=range(-255, 256))
 
     def __await__(self):
         return self.coro().__await__()
