@@ -582,12 +582,11 @@ class Runner:
         "Start the subprocess and assign to `self.proc`."
         with log_timer("asyncio.create_subprocess_exec"):
             sys.audit(AUDIT_EVENT_SUBPROCESS_SPAWN, opts.args[0])
-            if opts.pty_fds and _BSD:
-                pty_util.patch_child_watcher()
-            self._proc = await asyncio.create_subprocess_exec(
-                *opts.args,
-                **opts.kwd_args,
-            )
+            with pty_util.set_ignore_child_watcher(opts.pty_fds and _BSD):
+                self._proc = await asyncio.create_subprocess_exec(
+                    *opts.args,
+                    **opts.kwd_args,
+                )
 
     @log_method(LOG_DETAIL)
     async def _waiter(self):
@@ -755,11 +754,7 @@ class Runner:
             if failure:
                 info["failure"] = failure
             if phase == "signal":
-                if signal is None:
-                    info["signal"] = "SIGKILL"  # SIGKILL (even on win32)
-                else:
-                    # If signal name begins with "Signal.", elide the prefix.
-                    info["signal"] = str(signal).replace("Signals.", "")
+                info["signal"] = _signame(signal)
             callback(phase, info)
 
     def __repr__(self):
@@ -1046,3 +1041,13 @@ def _cleanup(command):
         open_fds.extend(command.options.pass_fds)
 
     close_fds(open_fds)
+
+
+def _signame(signal):
+    "Return string name of signal."
+    if signal is None:
+        return "SIGKILL"  # SIGKILL (even on win32)
+    try:
+        return signal.name
+    except AttributeError:
+        return str(signal)
